import java.lang.reflect.Method;

/**
 * Diese Klasse bietet verschiedene Methoden zum Umgang mit Beans .<br/>
 * Java-Beans sind Klassen mit set-get-Methoden fr die Membervariablen (Properties). <br/>
 * Es gibt Methoden zum Aufruf von Settern und Gettern mit und ohne Index. <br/>
 * Der Name der Property wird als String nicht kapitalisiert (kleiner Anfangsbuchstabe) bergeben. <br/>
 *
 * @author: Heiner Kcker
 * @version $Id: $
 */
public class BeanSetGet
{

  /**
   * Diese Methode dient zum Test ber BeanSetGet.bat
   *
   * @param args
   */
  public static void main(String[] args)
  {
    MyBean  myBean = new MyBean();

    // myBean.setMeldung( "test Meldung" ) ;
    set(myBean, "meldung", "test Meldung");
    System.out.println(get(myBean, "meldung"));
    set(myBean, "meldung", 1, "test indizierte Meldung");
    System.out.println(get(myBean, "meldung", 1));
    setIndexed(myBean, "meldung[34]", "test indizierte Meldung mit []");
    System.out.println(getIndexed(myBean, "meldung[34]"));
  } // end method main


  /**
   * Constructor declaration<br/>
   * Instanziieren nicht erlaubt
   */
  private BeanSetGet()
  {
    ;
  }// end constructor

  /**
   * set-Methode mit einfachem Parameter aufrufen. <br/>
   * setXy( wert-Objekt )
   *
   * @param bean      die Bean
   * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
   * @param value     zu setzender Wert als Objekt, Elementartypen mssen in die entsprechenden Klassen umgewandelt werden (Integer, Double, Boolean usw.)
   */
  public static void set(Object bean, String property, Object value)
  {
    try
    {
      String    setProp = "set" + upper1(property);
      Method[]  methodArray = bean.getClass().getMethods();

      for(int i = 0; i < methodArray.length; i++)
      {
        if(setProp.equals(methodArray[i].getName()))  // Namensgleichheit
        {
          // Prfen ob Paramterliste ok
          Class[] paramClassArray = methodArray[i].getParameterTypes();

          if(paramClassArray.length == 1)             // ein Parameter
          {
            if(paramClassArray[0].isInstance(value))  // ob Klasse passt
            {
              Object[]  paramArr = { value } ;

              methodArray[i].invoke(bean, paramArr);

              return;                                 // Auftrag erfllt
            }
          }
        }
      }
    }
    catch(Exception e)
    {
      System.out.println(e.getMessage()); // hier noch eigene Fehlerbehandlung einbauen
    }
  }//end method set

  /**
   * get-Methode mit einfachem Parameter aufrufen. <br/>
   * getXy()
   *
   * @param bean      die Bean
   * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
   * @return          abgefragter Wert als Objekt, Elementartypen werden in die entsprechenden Klassen umgewandelt (Integer, Double, Boolean usw.)
   */
  public static Object get(Object bean, String property)
  {
    try
    {
      String    getProp = "get" + upper1(property);
      Method[]  methodArray = bean.getClass().getMethods();

      for(int i = 0; i < methodArray.length; i++)
      {
        if(getProp.equals(methodArray[i].getName()))      // Namensgleichheit
        {
          // Prfen ob Paramterliste ok
          Class[] paramClassArray = methodArray[i].getParameterTypes();

          if(paramClassArray.length == 0)                 // kein Parameter
          {
            Object[]  paramArr = new Object[0];

            return methodArray[i].invoke(bean, paramArr); // Auftrag erfllt
          }
        }
      }
    }
    catch(Exception e)
    {
      System.out.println(e.getMessage()); // hier noch eigene Fehlerbehandlung einbauen
    }

    return null;
  }//end method get

  /**
   * set-Methode mit indiziertem Parameter aufrufen. <br/>
   * setXy( int-index , wert-Objekt )
   *
   * @param bean      die Bean
   * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
   * @param index     laufende Nummer
   * @param value     zu setzender Wert als Objekt, Elementartypen mssen in die entsprechenden Klassen umgewandelt werden (Integer, Double, Boolean usw.)
   */
  public static void set(Object bean, String property, int index, Object value)
  {
    try
    {
      String    setProp = "set" + upper1(property);
      Method[]  methodArray = bean.getClass().getMethods();

      for(int i = 0; i < methodArray.length; i++)
      {
        if(setProp.equals(methodArray[i].getName()))        // Namensgleichheit
        {
          // Prfen ob Paramterliste ok
          Class[] paramClassArray = methodArray[i].getParameterTypes();

          if(paramClassArray.length == 2)                   // zwei Parameter
          {
            if("int".equals(paramClassArray[0].getName()))  // erster Parameter ist Integer
            {
              if(paramClassArray[1].isInstance(value))      // ob Klasse passt
              {
                Object[]  paramArr = new Object[2];

                paramArr[0] = new Integer(index);
                paramArr[1] = value;

                methodArray[i].invoke(bean, paramArr);

                return;                                     // Auftrag erfllt
              }
            }
          }
        }
      }
    }
    catch(Exception e)
    {
      System.out.println(e.getMessage()); // hier noch eigene Fehlerbehandlung einbauen
    }
  }//end method set

  /**
   * get-Methode mit Index aufrufen. <br/>
   * getXy( int-index )
   *
   * @param bean      die Bean
   * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
   * @param index     laufende Nummer
   * @return          abgefragter Wert als Objekt, Elementartypen werden in die entsprechenden Klassen umgewandelt (Integer, Double, Boolean usw.)
   */
  public static Object get(Object bean, String property, int index)
  {
    try
    {
      String    getProp = "get" + upper1(property);
      Method[]  methodArray = bean.getClass().getMethods();

      for(int i = 0; i < methodArray.length; i++)
      {
        if(getProp.equals(methodArray[i].getName()))      // Namensgleichheit
        {
          // Prfen ob Paramterliste ok
          Class[] paramClassArray = methodArray[i].getParameterTypes();

          if(paramClassArray.length == 1)                 // ein Parameter
          {
            Object[]  paramArr = { new Integer(index) } ;

            return methodArray[i].invoke(bean, paramArr); // Auftrag erfllt
          }
        }
      }
    }
    catch(Exception e)
    {
      System.out.println(e.getMessage()); // hier noch eigene Fehlerbehandlung einbauen
    }

    return null;
  }//end method get

  /**
   * set-Methode mit indiziertem Parameter als "parameter[inx]" (z.B. name[3]) aufrufen. <br/>
   * die Methode setIndexed stellt automatisch fest, ob ein Index im Property-Namen enthalten ist. <br/>
   * Der Aufruf ohne Index ([nr]) ist also auch mglich. <br/>
   *
   * @param bean      die Bean
   * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
   * @param value     zu setzender Wert als Objekt, Elementartypen mssen in die entsprechenden Klassen umgewandelt werden (Integer, Double, Boolean usw.)
   */
  public static void setIndexed(Object bean, String property, Object value)
  {
    try
    {
      if(property.indexOf('[') > -1 && property.indexOf(']') > -1)
      {
        String  propStr = property.substring(0, property.indexOf('['));
        String  nrStr = property.substring(property.indexOf('[') + 1);

        nrStr = nrStr.substring(0, nrStr.indexOf(']'));

        int index = (new Integer(nrStr)).intValue();

        set(bean, propStr, index, value);
      }
      else
      {
        set(bean, property, value);
      }
    }
    catch(Exception e)
    {
      System.out.println(e.getMessage()); // hier noch eigene Fehlerbehandlung einbauen
    }
  }//end method setIndexed

  /**
   * get-Methode mit indiziertem Parameter als "parameter[inx]" (z.B. name[3]) aufrufen. <br/>
   * die Methode getIndexed stellt automatisch fest, ob ein Index im Property-Namen enthalten ist. <br/>
   * Der Aufruf ohne Index ([nr]) ist also auch mglich. <br/>
   *
   * @param bean      die Bean
   * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
   * @return          abgefragter Wert als Objekt, Elementartypen werden in die entsprechenden Klassen umgewandelt (Integer, Double, Boolean usw.)
   */
  public static Object getIndexed(Object bean, String property)
  {
    try
    {
      if(property.indexOf('[') > -1 && property.indexOf(']') > -1)
      {
        String  propStr = property.substring(0, property.indexOf('['));
        String  nrStr = property.substring(property.indexOf('[') + 1);

        nrStr = nrStr.substring(0, nrStr.indexOf(']'));

        int index = (new Integer(nrStr)).intValue();

        return get(bean, propStr, index);
      }
      else
      {
        return get(bean, property);
      }
    }
    catch(Exception e)
    {
      System.out.println(e.getMessage()); // hier noch eigene Fehlerbehandlung einbauen
    }

    return null;
  }//end method getIndexed

  /**
   * ersten Buchstaben im String gro machen, Capitalize
   *
   * @param strPaLower zu bearbeitender String
   * @return kapitalisierter String
   */
  public static String upper1(String strPaLower)
  {
    String  retString = null;

    if(strPaLower != null)
    {
      if(strPaLower.length() < 2)
      {
        retString = strPaLower.toUpperCase();
      }
      else
      {
        retString = strPaLower.substring(0, 1).toUpperCase();
        retString += strPaLower.substring(1);
      }
    }

    return retString;
  } // end method upper1

} // end class BeanSetGet
